package com.poc.alvachien.authserverdemo;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.oauth2.core.AuthorizationGrantType;
import org.springframework.security.oauth2.core.ClientAuthenticationMethod;
import org.springframework.security.oauth2.core.oidc.OidcScopes;
import org.springframework.security.oauth2.server.authorization.client.RegisteredClient;
import org.springframework.security.oauth2.server.authorization.settings.ClientSettings;

import com.poc.alvachien.authserverdemo.component.JpaRegisteredClientRepository;
import com.poc.alvachien.authserverdemo.model.UserAccount;
import com.poc.alvachien.authserverdemo.model.UserRole;
import com.poc.alvachien.authserverdemo.repository.UserAccountRepository;
import com.poc.alvachien.authserverdemo.repository.UserRoleRepository;

import java.util.UUID;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;

@SpringBootApplication
public class AuthServerDemoApplication {
	private static final Logger log = LoggerFactory.getLogger(AuthServerDemoApplication.class);

	public static void main(String[] args) {
		SpringApplication.run(AuthServerDemoApplication.class, args);
	}

	@Bean
	public CommandLineRunner preparetestdata(UserAccountRepository userRepository, UserRoleRepository roleRepository, JpaRegisteredClientRepository clientRepository,
		PasswordEncoder passwordEncoder) {
		return (args) -> {
			log.info("Entering CommandLineRunner");

			Long rolecnt = roleRepository.count();
			log.info("Role Count = " + rolecnt);

			if (rolecnt == 0) 
			{
				log.info("No role found, insert the default one");
				UserRole role = new UserRole();
				role.setName("STANDARD_ROLE");
				roleRepository.save(role);

				log.info("Test role 'STANDARD_ROLE' created");
				rolecnt = roleRepository.count();
				log.info("Role count = " + rolecnt);
			}

			Long userCnt = userRepository.count();
			log.info("User Count = " + userCnt);
			if (userCnt == 0) {
				log.info("No User found, insert the default one");
				UserAccount ua = new UserAccount();
				ua.setUsername("test");
				ua.setEmail("test@test.com");
				ua.setPassword(passwordEncoder.encode("test"));
				ua.setActive(true);

				// UserRole role = roleRepository.findByName("STANDARD_ROLE");
				// ua.setRoles(Arrays.asList(role));
				userRepository.save(ua);

				log.info("Test User 'test' with role 'STANDARD_ROLE' created");
				userCnt = userRepository.count();
				log.info("User Count = " + userCnt);
			}

			Long clientCnt = clientRepository.count();
			log.info("Client Count = " + userCnt);
			if (clientCnt == 0) {
				log.info("No Client found, insert the default one");
				RegisteredClient client = RegisteredClient.withId(UUID.randomUUID().toString())
				.clientId("angular-client")
				.clientSecret("{noop}secret")
				.clientAuthenticationMethod(ClientAuthenticationMethod.CLIENT_SECRET_BASIC)
				.authorizationGrantType(AuthorizationGrantType.AUTHORIZATION_CODE)
				.authorizationGrantType(AuthorizationGrantType.REFRESH_TOKEN)
				.authorizationGrantType(AuthorizationGrantType.CLIENT_CREDENTIALS)
				.redirectUri("http://127.0.0.1:8080/login/oauth2/code/messaging-client-oidc")
				.redirectUri("http://127.0.0.1:8080/authorized")
				.scope(OidcScopes.OPENID)
				.scope(OidcScopes.PROFILE)
				.scope("message.read")
				.scope("message.write")
				.clientSettings(ClientSettings.builder().requireAuthorizationConsent(true).build())
				.build();
				clientRepository.save(client);

				log.info("Test Client 'angular-client' created");
	
				clientCnt = clientRepository.count();
				log.info("Client Count = " + userCnt);
			}			
		};
	}
}
