package com.poc.alvachien.authserverdemo.controller;

import jakarta.validation.Valid;

import java.util.List;

import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import com.poc.alvachien.authserverdemo.model.LoginDto;
import com.poc.alvachien.authserverdemo.model.UserAccount;
import com.poc.alvachien.authserverdemo.model.UserAccountDto;
import com.poc.alvachien.authserverdemo.service.UserService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


@Controller
public class MainController {
	private static final Logger log = LoggerFactory.getLogger(MainController.class);
    private UserService userService;

    public MainController(UserService userService) {
        this.userService = userService;
    }

    @GetMapping("/")
    public String home(){
        log.info("Requiring / and will redirect to /home");

        return "home";
    }
    
    @GetMapping("/login")
    public String loginForm(Model model) {
        log.info("Requiring /login");

        LoginDto ld = new LoginDto();
        model.addAttribute("login", ld);
        return "login";
    }

    // @PostMapping("/login")
    // public String login(@Valid @ModelAttribute("login") LoginDto loginData,
    //         BindingResult result, Model model) {
    //     log.info("Posting to /login");
    //     //return sysUserService.login(loginBody);
    //     Authentication authentication = authenticationManager
    //             .authenticate(new UsernamePasswordAuthenticationToken(loginData.getEmail(), loginData.getPassword()));

    //     return "user";
    // }
    
    // handler method to handle user registration request
    @GetMapping("register")
    public String showRegistrationForm(Model model){
        log.info("Requiring /register");

        UserAccountDto user = new UserAccountDto();
        model.addAttribute("user", user);
        return "register";
    }

    // handler method to handle register user form submit request
    @PostMapping("/register/save")
    public String registration(@Valid @ModelAttribute("user") UserAccountDto user,
                               BindingResult result,
                               Model model){
        log.info("Posting to /register/save");

        UserAccount existing = userService.findByEmail(user.getEmail());
        if (existing != null) {
            result.rejectValue("email", null, "There is already an account registered with that email");
        }
        if (result.hasErrors()) {
            model.addAttribute("user", user);
            return "register";
        }
        userService.saveUser(user);
        return "redirect:/register?success";
    }


    @GetMapping("/user")
    public String currentUser(Model model) {
        log.info("Requiring /user");
        return "user";
    }

    @GetMapping("/admin")
    public String listRegisteredUsers(Model model){
        log.info("Requiring /admin");

        List<UserAccountDto> users = userService.findAllUsers();
        model.addAttribute("users", users);
        return "admin";
    }    
}
