package com.poc.alvachien.authserverdemo.service;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import com.poc.alvachien.authserverdemo.model.UserAccount;
import com.poc.alvachien.authserverdemo.model.UserAccountDto;
import com.poc.alvachien.authserverdemo.model.UserRole;
import com.poc.alvachien.authserverdemo.repository.UserAccountRepository;
import com.poc.alvachien.authserverdemo.repository.UserRoleRepository;

@Service
public class UserServiceImpl implements UserService {
    private UserAccountRepository userRepository;
    private UserRoleRepository roleRepository;
    private PasswordEncoder passwordEncoder;

    public UserServiceImpl(UserAccountRepository userRepository,
                            UserRoleRepository roleRepository,
                           PasswordEncoder passwordEncoder) {
        this.userRepository = userRepository;
        this.roleRepository = roleRepository;
        this.passwordEncoder = passwordEncoder;
    }

    @Override
    public void saveUser(UserAccountDto userDto) {
        UserAccount user = new UserAccount();
        user.setUsername(userDto.getFirstName() + " " + userDto.getLastName());
        user.setEmail(userDto.getEmail());

        //encrypt the password once we integrate spring security
        //user.setPassword(userDto.getPassword());
        user.setPassword(passwordEncoder.encode(userDto.getPassword()));
        UserRole role = roleRepository.findByName("ROLE_ADMIN");
        if(role == null){
            role = checkRoleExist();
        }
        user.setRoles(Arrays.asList(role));
        userRepository.save(user);
    }

    @Override
    public UserAccount findByEmail(String email) {
        return userRepository.findByEmail(email);
    }

    @Override
    public List<UserAccountDto> findAllUsers() {
        List<UserAccount> users = userRepository.findAll();
        return users.stream().map((user) -> convertEntityToDto(user))
                .collect(Collectors.toList());
    }

    private UserAccountDto convertEntityToDto(UserAccount user){
        UserAccountDto userDto = new UserAccountDto();
        String[] name = user.getUsername().split(" ");
        userDto.setFirstName(name[0]);
        userDto.setLastName(name[1]);
        userDto.setEmail(user.getEmail());
        return userDto;
    }

    private UserRole checkRoleExist() {
        UserRole role = new UserRole();
        role.setName("ROLE_ADMIN");
        return roleRepository.save(role);
    }
    
}
